@@ -23,25 +23,28 @@ output_container = "output"
 
 
 class TaskResult:
-    def __init__(self, result, error_message, stack_trace, offset):
+    def __init__(self, result, error_message, stack_trace, progress, state):
         self.result = result
         self.error_message = error_message
         self.stack_trace = stack_trace
-        self.offset = offset
+        self.progress = progress
+        self.state = state
 
     @staticmethod
     def create_from_request_entity(entity):
-        result, error, trace, offset = None, None, None, None
+        result, error, trace, progress, state = None, None, None, None, None
         if 'Result' in entity:
             result = entity['Result']
         if 'Error' in entity:
             error = entity['Error']
         if 'StackTrace' in entity:
             trace = entity['StackTrace']
-        if 'Offset' in entity:
-            offset = entity['Offset']
+        if 'Progress' in entity:
+            progress = entity['Progress']
+        if "State" in entity:
+            state = entity["State"]
         
-        return TaskResult(result, error, trace, offset)
+        return TaskResult(result, error, trace, progress, state)
 
     def to_json(self):
         json_response = {"Completed": False}
@@ -52,8 +55,10 @@ class TaskResult:
             json_response['Error'] = self.error_message
             json_response['StackTrace'] = self.stack_trace
             json_response['Completed'] = True
-        if self.offset:
-            json_response['Progress'] = self.offset
+        if self.progress:
+            json_response['Progress'] = self.progress
+        if self.state and self.state == "Completed":
+            json_response["Completed"] = True
         return json_response
 
 
@@ -94,6 +99,21 @@ class Worker:
         self._enqueue(request_id)
         self._wake_up()
         return request_id
+        
+    def get_result(self, request_id):
+        guid, request_hash = self._decode_id(request_id)
+        entity = self.table_client.get_entity(self.metadata_table_name, guid, request_hash)
+        if "ChildTasksPartition" in entity and entity['ChildTasksPartition']:
+            child_tasks_partition = entity['ChildTasksPartition']
+            completed_count = 0
+            total_count = 0
+            for task_entity in self.table_client.query_entities(self.metadata_table_name, filter="PartitionKey eq '%s'" % child_tasks_partition):
+                total_count += 1
+                if 'State' in task_entity and task_entity['State'] == 'Completed':
+                    completed_count += 1
+            entity["Progress"] = completed_count * self.segment_limit
+
+        return TaskResult.create_from_request_entity(entity)
 
     def process_messages_stateful(self, func, blob_func=None):
         state_row = list(self.table_client.query_entities(state_table))[0]
@@ -166,7 +186,11 @@ class Worker:
         
         serialized_result = "\n".join(json.dumps(r) for r in result)
         if whole_blob_read:
+            parts = blob_name.split("/")
+            if parts[0] == self.worker_name:
+                blob_name =  "/".join(parts[1:])
             out_blob = self._get_output_blob("%s/%s" % (self.worker_name, blob_name))
+
             out_blob.upload_blob(serialized_result, timeout=1200)
             request_entity['State'] = 'Completed'
             self.table_client.update_entity(self.metadata_table_name, request_entity)
@@ -189,23 +213,19 @@ class Worker:
                 completed_blobs.append(task_entity["BlobName"])
         if len(completed_blobs) == total_count:
             self._merge_blobs(completed_blobs, request_entity["BlobName"])
-            request_entity["State"] = "Completed"
-            self.table_client.update_entity(self.metadata_table_name, request_entity)
         else:
-            request_entity['Progress'] = len(completed_blobs) * self.segment_limit
-            self.table_client.update_entity(self.metadata_table_name, request_entity)
             self._enqueue(request_id)
 
     def _split_blob_into_chunks(self, segment_iterator, blob_name):
         blob_names = []
         batch = []
         batch_number = 1
-        for source_segment in segment_iterator:
-            batch.append(source_segment)
-            if len(batch) == self.segment_limit:
+        for source_segment, target_segment in segment_iterator:
+            batch.append([source_segment, target_segment])
+            if len(batch) == self.segment_limit - 1:
                 partial_blob_name = "%s/%s_%d" % (self.worker_name, blob_name, batch_number)
                 out_blob = self._get_input_blob(partial_blob_name)
-                serialized_result = json.dumps(batch)
+                serialized_result = "\n".join([json.dumps(pair) for pair in batch])
                 out_blob.upload_blob(serialized_result, timeout=1200)
                 blob_names.append(partial_blob_name)
                 batch = []
@@ -213,7 +233,7 @@ class Worker:
         if len(batch):
             partial_blob_name = "%s/%s_%d" % (self.worker_name, blob_name, batch_number)
             out_blob = self._get_input_blob(partial_blob_name)
-            serialized_result = json.dumps(batch)
+            serialized_result = "\n".join([json.dumps(pair) for pair in batch])
             out_blob.upload_blob(serialized_result, timeout=1200)
             blob_names.append(partial_blob_name)
         return blob_names
@@ -243,10 +263,11 @@ class Worker:
         return partition_key
 
     def _merge_blobs(self, blob_names, blob_name):
-        out_blob = self._get_output_blob("%s/%s" % (self.worker_name, blob_names))
+        out_blob = self._get_output_blob("%s/%s" % (self.worker_name, blob_name))
+        blob_names.append("%s/%s_0" % (self.worker_name, blob_name))
         content = []
         for partial_blob_name in sorted(blob_names, key=lambda b: int(b.split('_')[-1])):
-            in_blob = self._get_output_blob("%s/%s" % (self.worker_name, partial_blob_name))
+            in_blob = self._get_output_blob(partial_blob_name)
             blob_content = in_blob.download_blob().readall().decode()
             content.append(blob_content)
         out_blob.upload_blob("\n".join(content), timeout=1200)
@@ -262,7 +283,7 @@ class Worker:
         params = None
         if 'Parameters' in request_entity:
             params = json.loads(request_entity['Parameters'])
-        child_tasks_partition = 0
+        child_tasks_partition = None
         if 'ChildTasksPartition' in request_entity:
             child_tasks_partition = request_entity['ChildTasksPartition']
         retry_count = 0
@@ -277,7 +298,7 @@ class Worker:
         whole_blob_read = True
         segments_processed = 0
         blobs_iterator = self._read_blob_by_segments(in_blob)
-        for source_segment in blobs_iterator:
+        for source_segment, target_segment in blobs_iterator:
             batch.append(source_segment)
             if len(batch) == self.batch_size:
                 result.extend(func(batch, params))
@@ -303,11 +324,6 @@ class Worker:
         request_entity['Error'] = error_message
         request_entity['StackTrace'] = trace
         self.table_client.update_entity(self.metadata_table_name, request_entity)
-        
-    def get_result(self, request_id):
-        guid, request_hash = self._decode_id(request_id)
-        entity = self.table_client.get_entity(self.metadata_table_name, guid, request_hash)
-        return TaskResult.create_from_request_entity(entity)
 
     def _insert_request_metadata(self, data):
         serialized_body = json.dumps(data)
@@ -371,25 +387,25 @@ class Worker:
         blob_data = input_blob.download_blob()
         last_piece = ""
         for chunk in blob_data.chunks():
-            lines = [l.strip() for l in chunk.decode().split("\r\n") if len(l.strip()) > 0]
+            lines = [l.strip() for l in chunk.decode().splitlines() if len(l.strip()) > 0]
             lines[0] = last_piece + lines[0]
             last_piece = lines[-1]
             for l in lines[:-1]:
                 l = unicodedata.normalize("NFKD", l).strip()
                 if len(l) == 0:
                     continue
-                source_segment = json.loads(l)[0].strip()
-                yield source_segment
+                source_segment, target_language = json.loads(l)
+                yield source_segment.strip(), target_language.strip()
 
         last_piece = unicodedata.normalize("NFKD", last_piece).strip()
         if len(last_piece) > 0:
             source = None
             try:
-                source = json.loads(last_piece)[0].strip()
+                source, target = json.loads(last_piece)
             except:
                 pass
             if source:
-                yield source
+                yield source.strip(), target.strip()
 
     @staticmethod
     def _generate_id(data):
